---------------------------------------------------------------------------
--                                                                       --
--  ast2dot is an ASIS application that outputs the AST in dot format.   --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--          Copyright (c) 2006, Raul Fechete, Georg Kienesberger         --
--                                                                       --
---------------------------------------------------------------------------
-- This file is part of ast2dot.                                         --
--                                                                       --
-- ast2dot is free software; you can redistribute it and/or modify       --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 2 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- ast2dot is distributed in the hope that it will be useful,            --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with ast2dot; if not, write to the Free Software                --
-- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 --
-- USA                                                                   --
--                                                                       --
---------------------------------------------------------------------------

with Asis; use Asis;
with Asis.Elements;
with Asis.Text; use Asis.Text;
with Asis.Ids;

with Ada.Wide_Text_IO; use Ada.Wide_Text_IO;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Ada.Strings.Fixed;
with Ada.Strings.Maps;
with Ada.Characters.Handling; use Ada.Characters.Handling;
with Ada.Characters.Latin_1;
with Ada.Exceptions; use Ada.Exceptions;
with Ada.Directories;

with GNAT.OS_Lib;

with Level_List; use Level_List;
with Options;

package body Process_Output is

   procedure BeginDigraph (Name: in String) is
   begin

      if To_String(Options.Dir) /= "" then
         Filename := Options.Dir & To_Unbounded_String("/" & Name);

         for i in 1..Length(Filename) loop
            if Ada.Strings.Unbounded.Element(Filename, i) = ' ' then
               Replace_Element(Filename, i, '_');
            end if;
         end loop;

         declare
            i: Positive := 1;
         begin
            if Ada.Directories.Exists(To_String(Filename) & ".dot") then
               while Ada.Directories.Exists(To_String(Filename) & "." & Ada.Strings.Fixed.Trim(Positive'Image(i), Ada.Strings.Both) & ".dot") loop
                  i := i + 1;
               end loop;

               Append(Filename, "." & Ada.Strings.Fixed.Trim(Positive'Image(i), Ada.Strings.Both) & ".dot");
            else
               Append(Filename, ".dot");
            end if;

            Create(File=>Cur_File, Mode=>Out_File, Name=>To_String(Filename));
         exception
            when others =>
               Put_Line(Standard_Error, "Unable to create file: " & To_Wide_String(To_String(Filename)));
               GNAT.OS_Lib.OS_Exit(1);
         end;
         Set_Output(Cur_File);
      end if;

      Put_Line("digraph """ & To_Wide_String(Name) & """ {");
      Put_Line("    node [shape=box];");
      New_Line;

      if To_String(Options.Dir) /= "" then
         Set_Output(Standard_Output);
      end if;
   end BeginDigraph;

   procedure EndDigraph is
   begin

      if To_String(Options.Dir) /= "" then
         Set_Output(Cur_File);
      end if;

      Put_Line("}");
      New_Line;

      if To_String(Options.Dir) /= "" then
         Close(Cur_File);
         Set_Output(Standard_Output);
      end if;
   end EndDigraph;

   procedure PrintDot (MyElement: in Asis.Element; MyLevel: in Natural; MyLastLevel: in Natural; MyId: in Natural; MyLevelList: in out LevelList) is

      Argument_Element_Kind: Asis.Element_Kinds;
      Argument_Pragma_Kind: Asis.Pragma_Kinds;
      Argument_DefName_Kind: Asis.Defining_Name_Kinds;
      Argument_Declaration_Kind: Asis.Declaration_Kinds;
      Argument_Definition_Kind: Asis.Definition_Kinds;
      Argument_Expression_Kind: Asis.Expression_Kinds;
      Argument_Association_Kind: Asis.Association_Kinds;
      Argument_Statement_Kind: Asis.Statement_Kinds;
      Argument_Path_Kind: Asis.Path_Kinds;
      Argument_Clause_Kind: Asis.Clause_Kinds;

      MyLines: Asis.Text.Line_List := Lines (MyElement);
      MyTextAux: Unbounded_String := To_Unbounded_String("");

   begin

      if To_String(Options.Dir) /= "" then
         Set_Output(Cur_File);
      end if;

      Put("    ""N");
      Put(To_Wide_String(Ada.Strings.Fixed.Trim(Positive'Image(MyId), Ada.Strings.Both)));
      Put(""" [label=""");
      Put("N");
      Put(To_Wide_String(Ada.Strings.Fixed.Trim(Positive'Image(MyId), Ada.Strings.Both)));
      Put(": ");

      -- Try to identify the type of the element
      Argument_Element_Kind := Asis.Elements.Element_Kind (MyElement);
      case Argument_Element_Kind is
         when Not_An_Element =>            -- Nil_Element
            Put("Not_An_Element");
         when A_Pragma =>                  -- Asis.Elements
            Put("A_Pragma");
            Put(" SubType: ");
            Argument_Pragma_Kind := Asis.Elements.Pragma_Kind (MyElement);
            case Argument_Pragma_Kind is
               when Not_A_Pragma =>
                  Put("Not_A_Pragma");
               when An_All_Calls_Remote_Pragma =>
                  Put("An_All_Calls_Remote_Pragma");
               when An_Asynchronous_Pragma =>
                  Put("An_Asynchronous_Pragma");
               when An_Atomic_Pragma =>
                  Put("An_Atomic_Pragma");
               when An_Atomic_Components_Pragma =>
                  Put("An_Atomic_Components_Pragma");
               when An_Attach_Handler_Pragma =>
                  Put("An_Attach_Handler_Pragma");
               when A_Controlled_Pragma =>
                  Put("A_Controlled_Pragma");
               when A_Convention_Pragma =>
                  Put("A_Convention_Pragma");
               when A_Discard_Names_Pragma =>
                  Put("A_Discard_Names_Pragma");
               when An_Elaborate_Pragma =>
                  Put("An_Elaborate_Pragma");
               when An_Elaborate_All_Pragma =>
                  Put("An_Elaborate_All_Pragma");
               when An_Elaborate_Body_Pragma =>
                  Put("An_Elaborate_Body_Pragma");
               when An_Export_Pragma =>
                  Put("An_Export_Pragma");
               when An_Import_Pragma =>
                  Put("An_Import_Pragma");
               when An_Inline_Pragma =>
                  Put("An_Inline_Pragma");
               when An_Inspection_Point_Pragma =>
                  Put("An_Inspection_Point_Pragma");
               when An_Interrupt_Handler_Pragma =>
                  Put("An_Interrupt_Handler_Pragma");
               when An_Interrupt_Priority_Pragma =>
                  Put("An_Interrupt_Priority_Pragma");
               when A_Linker_Options_Pragma =>
                  Put("A_Linker_Options_Pragma");
               when A_List_Pragma =>
                  Put("A_List_Pragma");
               when A_Locking_Policy_Pragma =>
                  Put("A_Locking_Policy_Pragma");
               when A_Normalize_Scalars_Pragma =>
                  Put("A_Normalize_Scalars_Pragma");
               when An_Optimize_Pragma =>
                  Put("An_Optimize_Pragma");
               when A_Pack_Pragma =>
                  Put("A_Pack_Pragma");
               when A_Page_Pragma =>
                  Put("A_Page_Pragma");
               when A_Preelaborate_Pragma =>
                  Put("A_Preelaborate_Pragma");
               when A_Priority_Pragma =>
                  Put("A_Priority_Pragma");
               when A_Pure_Pragma =>
                  Put("A_Pure_Pragma");
               when A_Queuing_Policy_Pragma =>
                  Put("A_Queuing_Policy_Pragma");
               when A_Remote_Call_Interface_Pragma =>
                  Put("A_Remote_Call_Interface_Pragma");
               when A_Remote_Types_Pragma =>
                  Put("A_Remote_Types_Pragma");
               when A_Restrictions_Pragma =>
                  Put("A_Restrictions_Pragma");
               when A_Reviewable_Pragma =>
                  Put("A_Reviewable_Pragma");
               when A_Shared_Passive_Pragma =>
                  Put("A_Shared_Passive_Pragma");
               when A_Storage_Size_Pragma =>
                  Put("A_Storage_Size_Pragma");
               when A_Suppress_Pragma =>
                  Put("A_Suppress_Pragma");
               when A_Task_Dispatching_Policy_Pragma =>
                  Put("A_Task_Dispatching_Policy_Pragma");
               when A_Volatile_Pragma =>
                  Put("A_Volatile_Pragma");
               when A_Volatile_Components_Pragma =>
                  Put("A_Volatile_Components_Pragma");
               when An_Implementation_Defined_Pragma =>
                  Put("An_Implementation_Defined_Pragma");
               when An_Unknown_Pragma =>
                  Put("An_Unknown_Pragma");
               when others =>
                  Put("Other_Pragma");
            end case;
         when A_Defining_Name =>           -- Asis.Declarations
            Put("A_Defining_Name");
            Put(" SubType: ");
            Argument_DefName_Kind := Asis.Elements.Defining_Name_Kind (MyElement);
            case Argument_DefName_Kind is
               when Not_A_Defining_Name =>
                  Put("Not_A_Defining_Name");
               when A_Defining_Identifier =>
                  Put("A_Defining_Identifier");
               when A_Defining_Character_Literal =>
                  Put("A_Defining_Character_Literal");
               when A_Defining_Enumeration_Literal =>
                  Put("A_Defining_Enumeration_Literal");
               when A_Defining_Operator_Symbol =>
                  Put("A_Defining_Operator_Symbol");
               when A_Defining_Expanded_Name =>
                  Put("A_Defining_Expanded_Name");
            end case;
         when A_Declaration =>             -- Asis.Declarations
            Put("A_Declaration");
            Put(" SubType: ");
            Argument_Declaration_Kind := Asis.Elements.Declaration_Kind (MyElement);
            case Argument_Declaration_Kind is
               when Not_A_Declaration =>
                  Put("Not_A_Declaration");
               when An_Ordinary_Type_Declaration =>
                  Put("An_Ordinary_Type_Declaration");
               when A_Task_Type_Declaration =>
                  Put("A_Task_Type_Declaration");
               when A_Protected_Type_Declaration =>
                  Put("A_Protected_Type_Declaration");
               when An_Incomplete_Type_Declaration =>
                  Put("An_Incomplete_Type_Declaration");
               when A_Private_Type_Declaration =>
                  Put("A_Private_Type_Declaration");
               when A_Private_Extension_Declaration =>
                  Put("A_Private_Extension_Declaration");
               when A_Subtype_Declaration =>
                  Put("A_Subtype_Declaration");
               when A_Variable_Declaration =>
                  Put("A_Variable_Declaration");
               when A_Constant_Declaration =>
                  Put("A_Constant_Declaration");
               when A_Deferred_Constant_Declaration =>
                  Put("A_Deferred_Constant_Declaration");
               when A_Single_Task_Declaration =>
                  Put("A_Single_Task_Declaration");
               when A_Single_Protected_Declaration =>
                  Put("A_Single_Protected_Declaration");
               when An_Integer_Number_Declaration =>
                  Put("An_Integer_Number_Declaration");
               when A_Real_Number_Declaration =>
                  Put("A_Real_Number_Declaration");
               when An_Enumeration_Literal_Specification =>
                  Put("An_Enumeration_Literal_Specification");
               when A_Discriminant_Specification =>
                  Put("A_Discriminant_Specification");
               when A_Component_Declaration =>
                  Put("A_Component_Declaration");
               when A_Loop_Parameter_Specification =>
                  Put("A_Loop_Parameter_Specification");
               when A_Procedure_Declaration =>
                  Put("A_Procedure_Declaration");
               when A_Function_Declaration =>
                  Put("A_Function_Declaration");
               when A_Parameter_Specification =>
                  Put("A_Parameter_Specification");
               when A_Procedure_Body_Declaration =>
                  Put("A_Procedure_Body_Declaration");
               when A_Function_Body_Declaration =>
                  Put("A_Function_Body_Declaration");
               when A_Package_Declaration =>
                  Put("A_Package_Declaration");
               when A_Package_Body_Declaration =>
                  Put("A_Package_Body_Declaration");
               when An_Object_Renaming_Declaration =>
                  Put("An_Object_Renaming_Declaration");
               when An_Exception_Renaming_Declaration =>
                  Put("An_Exception_Renaming_Declaration");
               when A_Package_Renaming_Declaration =>
                  Put("A_Package_Renaming_Declaration");
               when A_Procedure_Renaming_Declaration =>
                  Put("A_Procedure_Renaming_Declaration");
               when A_Function_Renaming_Declaration =>
                  Put("A_Function_Renaming_Declaration");
               when A_Generic_Package_Renaming_Declaration =>
                  Put("A_Generic_Package_Renaming_Declaration");

               when A_Generic_Procedure_Renaming_Declaration =>
                  Put("A_Generic_Procedure_Renaming_Declaration");

               when A_Generic_Function_Renaming_Declaration =>
                  Put("A_Generic_Function_Renaming_Declaration");

               when A_Task_Body_Declaration =>
                  Put("A_Task_Body_Declaration");
               when A_Protected_Body_Declaration =>
                  Put("A_Protected_Body_Declaration");
               when An_Entry_Declaration =>
                  Put("An_Entry_Declaration");
               when An_Entry_Body_Declaration =>
                  Put("An_Entry_Body_Declaration");
               when An_Entry_Index_Specification =>
                  Put("An_Entry_Index_Specification");
               when A_Procedure_Body_Stub =>
                  Put("A_Procedure_Body_Stub");
               when A_Function_Body_Stub =>
                  Put("A_Function_Body_Stub");
               when A_Package_Body_Stub =>
                  Put("A_Package_Body_Stub");
               when A_Task_Body_Stub =>
                  Put("A_Task_Body_Stub");
               when A_Protected_Body_Stub =>
                  Put("A_Protected_Body_Stub");
               when An_Exception_Declaration =>
                  Put("An_Exception_Declaration");
               when A_Choice_Parameter_Specification =>
                  Put("A_Choice_Parameter_Specification");
               when A_Generic_Procedure_Declaration =>
                  Put("A_Generic_Procedure_Declaration");
               when A_Generic_Function_Declaration =>
                  Put("A_Generic_Function_Declaration");
               when A_Generic_Package_Declaration =>
                  Put("A_Generic_Package_Declaration");
               when A_Package_Instantiation =>
                  Put("A_Package_Instantiation");
               when A_Procedure_Instantiation =>
                  Put("A_Procedure_Instantiation");
               when A_Function_Instantiation =>
                  Put("A_Function_Instantiation");
               when A_Formal_Object_Declaration =>
                  Put("A_Formal_Object_Declaration");
               when A_Formal_Type_Declaration =>
                  Put("A_Formal_Type_Declaration");
               when A_Formal_Procedure_Declaration =>
                  Put("A_Formal_Procedure_Declaration");
               when A_Formal_Function_Declaration =>
                  Put("A_Formal_Function_Declaration");
               when A_Formal_Package_Declaration =>
                  Put("A_Formal_Package_Declaration");
               when A_Formal_Package_Declaration_With_Box =>
                  Put("A_Formal_Package_Declaration_With_Box");
               when others =>
                  Put("Other_Declaration");
            end case;
         when A_Definition =>              -- Asis.Definitions
            Put("A_Definition");
            Put(" SubType: ");
            Argument_Definition_Kind := Asis.Elements.Definition_Kind (MyElement);
            case Argument_Definition_Kind is
               when Not_A_Definition =>
                  Put("Not_A_Definition");
               when A_Type_Definition =>
                  Put("A_Type_Definition");
               when A_Subtype_Indication =>
                  Put("A_Subtype_Indication");
               when A_Constraint =>
                  Put("A_Constraint");
               when A_Component_Definition =>
                  Put("A_Component_Definition");
               when A_Discrete_Subtype_Definition =>
                  Put("A_Discrete_Subtype_Definition");
               when A_Discrete_Range =>
                  Put("A_Discrete_Range");
               when An_Unknown_Discriminant_Part =>
                  Put("An_Unknown_Discriminant_Part");
               when A_Known_Discriminant_Part =>
                  Put("A_Known_Discriminant_Part");
               when A_Record_Definition =>
                  Put("A_Record_Definition");
               when A_Null_Record_Definition =>
                  Put("A_Null_Record_Definition");
               when A_Null_Component =>
                  Put("A_Null_Component");
               when A_Variant_Part =>
                  Put("A_Variant_Part");
               when A_Variant =>
                  Put("A_Variant");
               when An_Others_Choice =>
                  Put("An_Others_Choice");
               when A_Private_Type_Definition =>
                  Put("A_Private_Type_Definition");
               when A_Tagged_Private_Type_Definition =>
                  Put("A_Tagged_Private_Type_Definition");
               when A_Private_Extension_Definition =>
                  Put("A_Private_Extension_Definition");
               when A_Task_Definition =>
                  Put("A_Task_Definition");
               when A_Protected_Definition =>
                  Put("A_Protected_Definition");
               when A_Formal_Type_Definition =>
                  Put("A_Formal_Type_Definition");
               when others =>
                  Put("Other_Definition");
            end case;
         when An_Expression =>             -- Asis.Expressions
            Put("An_Expression");
            Put(" SubType: ");
            Argument_Expression_Kind := Asis.Elements.Expression_Kind (MyElement);
            case Argument_Expression_Kind is
               when Not_An_Expression =>
                  Put("Not_An_Expression");
               when An_Integer_Literal =>
                  Put("An_Integer_Literal");
               when A_Real_Literal =>
                  Put("A_Real_Literal");
               when A_String_Literal =>
                  Put("A_String_Literal");
               when An_Identifier =>
                  Put("An_Identifier");
               when An_Operator_Symbol =>
                  Put("An_Operator_Symbol");
               when A_Character_Literal =>
                  Put("A_Character_Literal");
               when An_Enumeration_Literal =>
                  Put("An_Enumeration_Literal");
               when An_Explicit_Dereference =>
                  Put("An_Explicit_Dereference");
               when A_Function_Call =>
                  Put("A_Function_Call");
               when An_Indexed_Component =>
                  Put("An_Indexed_Component");
               when A_Slice =>
                  Put("A_Slice");
               when A_Selected_Component =>
                  Put("A_Selected_Component");
               when An_Attribute_Reference =>
                  Put("An_Attribute_Reference");
               when A_Record_Aggregate =>
                  Put("A_Record_Aggregate");
               when An_Extension_Aggregate =>
                  Put("An_Extension_Aggregate");
               when A_Positional_Array_Aggregate =>
                  Put("A_Positional_Array_Aggregate");
               when A_Named_Array_Aggregate =>
                  Put("A_Named_Array_Aggregate");
               when An_And_Then_Short_Circuit =>
                  Put("An_And_Then_Short_Circuit");
               when An_Or_Else_Short_Circuit =>
                  Put("An_Or_Else_Short_Circuit");
               when An_In_Range_Membership_Test =>
                  Put("An_In_Range_Membership_Test");
               when A_Not_In_Range_Membership_Test =>
                  Put("A_Not_In_Range_Membership_Test");
               when An_In_Type_Membership_Test =>
                  Put("An_In_Type_Membership_Test");
               when A_Not_In_Type_Membership_Test =>
                  Put("A_Not_In_Type_Membership_Test");
               when A_Null_Literal =>
                  Put("A_Null_Literal");
               when A_Parenthesized_Expression =>
                  Put("A_Parenthesized_Expression");
               when A_Type_Conversion =>
                  Put("A_Type_Conversion");
               when A_Qualified_Expression =>
                  Put("A_Qualified_Expression");
               when An_Allocation_From_Subtype =>
                  Put("An_Allocation_From_Subtype");
               when An_Allocation_From_Qualified_Expression =>
                  Put("An_Allocation_From_Qualified_Expression");

            end case;
         when An_Association =>            -- Asis.Expressions
            Put("An_Association");
            Put(" SubType: ");
            Argument_Association_Kind := Asis.Elements.Association_Kind (MyElement);
            case Argument_Association_Kind is
               when Not_An_Association =>
                  Put("Not_An_Association");
               when A_Pragma_Argument_Association =>
                  Put("A_Pragma_Argument_Association");
               when A_Discriminant_Association =>
                  Put("A_Discriminant_Association");
               when A_Record_Component_Association =>
                  Put("A_Record_Component_Association");
               when An_Array_Component_Association =>
                  Put("An_Array_Component_Association");
               when A_Parameter_Association =>
                  Put("A_Parameter_Association");
               when A_Generic_Association =>
                  Put("A_Generic_Association");
               when others =>
                  Put("Other_Association");
            end case;
         when A_Statement =>               -- Asis.Statements
            Put("A_Statement");
            Put(" SubType: ");
            Argument_Statement_Kind := Asis.Elements.Statement_Kind (MyElement);
            case Argument_Statement_Kind is
              when Not_A_Statement =>
                  Put("Not_A_Statement");
               when A_Null_Statement =>
                  Put("A_Null_Statement");
               when An_Assignment_Statement =>
                  Put("An_Assignment_Statement");
               when An_If_Statement =>
                  Put("An_If_Statement");
               when A_Case_Statement =>
                  Put("A_Case_Statement");
               when A_Loop_Statement =>
                  Put("A_Loop_Statement");
               when A_While_Loop_Statement =>
                  Put("A_While_Loop_Statement");
               when A_For_Loop_Statement =>
                  Put("A_For_Loop_Statement");
               when A_Block_Statement =>
                  Put("A_Block_Statement");
               when An_Exit_Statement =>
                  Put("An_Exit_Statement");
               when A_Goto_Statement =>
                  Put("A_Goto_Statement");
               when A_Procedure_Call_Statement =>
                  Put("A_Procedure_Call_Statement");
               when A_Return_Statement =>
                  Put("A_Return_Statement");
               when An_Accept_Statement =>
                  Put("An_Accept_Statement");
               when An_Entry_Call_Statement =>
                  Put("An_Entry_Call_Statement");
               when A_Requeue_Statement =>
                  Put("A_Requeue_Statement");
               when A_Requeue_Statement_With_Abort =>
                  Put("A_Requeue_Statement_With_Abort");
               when A_Delay_Until_Statement =>
                  Put("A_Delay_Until_Statement");
               when A_Delay_Relative_Statement =>
                  Put("A_Delay_Relative_Statement");
               when A_Terminate_Alternative_Statement =>
                  Put("A_Terminate_Alternative_Statement");
               when A_Selective_Accept_Statement =>
                  Put("A_Selective_Accept_Statement");
               when A_Timed_Entry_Call_Statement =>
                  Put("A_Timed_Entry_Call_Statement");
               when A_Conditional_Entry_Call_Statement =>
                  Put("A_Conditional_Entry_Call_Statement");
               when An_Asynchronous_Select_Statement =>
                  Put("An_Asynchronous_Select_Statement");
               when An_Abort_Statement =>
                  Put("An_Abort_Statement");
               when A_Raise_Statement =>
                  Put("A_Raise_Statement");
               when A_Code_Statement =>
                  Put("A_Code_Statement");
               when others =>
                  Put("Other_Statement");
            end case;
         when A_Path =>                    -- Asis.Statements
            Put("A_Path");
            Put(" SubType: ");
            Argument_Path_Kind := Asis.Elements.Path_Kind (MyElement);
            case Argument_Path_Kind is
               when Not_A_Path =>
                  Put("Not_A_Path");
               when An_If_Path =>
                  Put("An_If_Path");
               when An_Elsif_Path =>
                  Put("An_Elsif_Path");
               when An_Else_Path =>
                  Put("An_Else_Path");
               when A_Case_Path =>
                  Put("A_Case_Path");
               when A_Select_Path =>
                  Put("A_Select_Path");
               when An_Or_Path =>
                  Put("An_Or_Path");
               when A_Then_Abort_Path =>
                  Put("A_Then_Abort_Path");
            end case;
         when A_Clause =>                  -- Asis.Clauses
            Put("A_Clause");
            Put(" SubType: ");
            Argument_Clause_Kind := Asis.Elements.Clause_Kind (MyElement);
            case Argument_Clause_Kind is
               when Not_A_Clause =>
                  Put("Not_A_Clause");
               when A_Use_Package_Clause =>
                  Put("A_Use_Package_Clause");
               when A_Use_Type_Clause =>
                  Put("A_Use_Type_Clause");
               when A_With_Clause =>
                  Put("A_With_Clause");
               when A_Representation_Clause =>
                  Put("A_Representation_Clause");
               when A_Component_Clause =>
                  Put("A_Component_Clause");
            end case;
         when An_Exception_Handler =>     -- Asis.Statements
            Put("An_Exception_Handler");
      end case;

      Put("\n\n");

      -- Print each of the block lines
      for l in MyLines'Range loop

         -- trim whitespaces
         MyTextAux :=  To_Unbounded_String(
                         Ada.Strings.Fixed.Trim(To_String(Line_Image(MyLines(l))),
                                                Ada.Strings.Maps.To_Set(" " & Ada.Characters.Latin_1.HT),
                                                Ada.Strings.Maps.To_Set(" " & Ada.Characters.Latin_1.HT)));

         -- in dot format "s need to be masked with \
         declare
            i: Positive := 1;
         begin
            while i <= Length(MyTextAux) loop
               if Ada.Strings.Unbounded.Element(MyTextAux, i) = '"' then
                  Replace_Slice(MyTextAux, i, i, "\""");
                  i := i + 1;
               end if;
               i := i + 1;
            end loop;
         end;

         Put(To_Wide_String(To_String(MyTextAux)));
         Put("\n");

      end loop;

      Put("""];");
      New_Line;

      -- print edge
      declare
         ParentId: Natural;
      begin
         Get(MyLevelList, MyLastLevel, ParentId);

         Put("    ""N");
         Put(To_Wide_String(Ada.Strings.Fixed.Trim(Positive'Image(ParentId), Ada.Strings.Both)));
         Put(""" -> ""N");
         Put(To_Wide_String(Ada.Strings.Fixed.Trim(Positive'Image(MyId), Ada.Strings.Both)));
         Put(""";");
         New_Line;
      exception
         when Unset => null;
      end;

      if To_String(Options.Dir) /= "" then
         Set_Output(Standard_Output);
      end if;
   end PrintDot;

end Process_Output;
